@@ -165,7 +165,7 @@ cbz_load_page(fz_context *ctx, cbz_document *doc, int number)
 
 	fz_try(ctx)
 	{
-		page = fz_new_page(ctx, sizeof *page);
+		page = fz_new_derived_page(ctx, cbz_page);
 		page->super.bound_page = (fz_page_bound_page_fn *)cbz_bound_page;
 		page->super.run_page_contents = (fz_page_run_page_contents_fn *)cbz_run_page;
 		page->super.drop_page = (fz_page_drop_page_fn *)cbz_drop_page;
@@ -192,12 +192,12 @@ cbz_lookup_metadata(fz_context *ctx, cbz_document *doc, const char *key, char *b
 	return -1;
 }
 
-static cbz_document *
+static fz_document *
 cbz_open_document_with_stream(fz_context *ctx, fz_stream *file)
 {
 	cbz_document *doc;
 
-	doc = fz_new_document(ctx, cbz_document);
+	doc = fz_new_derived_document(ctx, cbz_document);
 
 	doc->super.drop_document = (fz_document_drop_fn *)cbz_drop_document;
 	doc->super.count_pages = (fz_document_count_pages_fn *)cbz_count_pages;
@@ -214,7 +214,7 @@ cbz_open_document_with_stream(fz_context *ctx, fz_stream *file)
 		fz_drop_document(ctx, &doc->super);
 		fz_rethrow(ctx);
 	}
-	return doc;
+	return &doc->super;
 }
 
 static int
@@ -238,7 +238,7 @@ cbz_recognize(fz_context *ctx, const char *magic)
 
 fz_document_handler cbz_document_handler =
 {
-	(fz_document_recognize_fn *)&cbz_recognize,
-	(fz_document_open_fn *)NULL,
-	(fz_document_open_with_stream_fn *)&cbz_open_document_with_stream
+	cbz_recognize,
+	NULL,
+	cbz_open_document_with_stream
 };
