@@ -310,7 +310,7 @@ pbm_write_header(fz_context *ctx, fz_band_writer *writer)
 	int w = writer->w;
 	int h = writer->h;
 
-	fz_printf(ctx, out, "P4\n%d %d\n", w, h);
+	fz_write_printf(ctx, out, "P4\n%d %d\n", w, h);
 }
 
 static void
@@ -320,29 +320,47 @@ pkm_write_header(fz_context *ctx, fz_band_writer *writer)
 	int w = writer->w;
 	int h = writer->h;
 
-	fz_printf(ctx, out, "P7\nWIDTH %d\nHEIGHT %d\nDEPTH 4\nMAXVAL 255\nTUPLTYPE CMYK\nENDHDR\n", w, h);
+	fz_write_printf(ctx, out, "P7\nWIDTH %d\nHEIGHT %d\nDEPTH 4\nMAXVAL 255\nTUPLTYPE CMYK\nENDHDR\n", w, h);
 }
 
 void
 fz_write_bitmap_as_pbm(fz_context *ctx, fz_output *out, fz_bitmap *bitmap)
 {
-	fz_band_writer *writer = fz_new_pbm_band_writer(ctx, out);
+	fz_band_writer *writer;
 
-	fz_write_header(ctx, writer, bitmap->w, bitmap->h, 1, 0, 0, 0, 1);
-	fz_write_band(ctx, writer, bitmap->stride, 0, bitmap->h, bitmap->samples);
-	fz_write_trailer(ctx, writer);
-	fz_drop_band_writer(ctx, writer);
+	if (bitmap->n != 1)
+		fz_throw(ctx, FZ_ERROR_GENERIC, "bitmap must be monochrome to save as PBM");
+
+	writer = fz_new_pbm_band_writer(ctx, out);
+	fz_try(ctx)
+	{
+		fz_write_header(ctx, writer, bitmap->w, bitmap->h, 1, 0, 0, 0, 0);
+		fz_write_band(ctx, writer, bitmap->stride, bitmap->h, bitmap->samples);
+	}
+	fz_always(ctx)
+		fz_drop_band_writer(ctx, writer);
+	fz_catch(ctx)
+		fz_rethrow(ctx);
 }
 
 void
 fz_write_bitmap_as_pkm(fz_context *ctx, fz_output *out, fz_bitmap *bitmap)
 {
-	fz_band_writer *writer = fz_new_pkm_band_writer(ctx, out);
+	fz_band_writer *writer;
 
-	fz_write_header(ctx, writer, bitmap->w, bitmap->h, 1, 0, 0, 0, 1);
-	fz_write_band(ctx, writer, bitmap->stride, 0, bitmap->h, bitmap->samples);
-	fz_write_trailer(ctx, writer);
-	fz_drop_band_writer(ctx, writer);
+	if (bitmap->n != 4)
+		fz_throw(ctx, FZ_ERROR_GENERIC, "bitmap must be CMYK to save as PKM");
+
+	writer = fz_new_pkm_band_writer(ctx, out);
+	fz_try(ctx)
+	{
+		fz_write_header(ctx, writer, bitmap->w, bitmap->h, 4, 0, 0, 0, 0);
+		fz_write_band(ctx, writer, bitmap->stride, bitmap->h, bitmap->samples);
+	}
+	fz_always(ctx)
+		fz_drop_band_writer(ctx, writer);
+	fz_catch(ctx)
+		fz_rethrow(ctx);
 }
 
 static void
@@ -353,14 +371,19 @@ pbm_write_band(fz_context *ctx, fz_band_writer *writer, int stride, int band_sta
 	int h = writer->h;
 	int n = writer->n;
 	int bytestride;
+	int end = band_start + band_height;
 
 	if (n != 1)
 		fz_throw(ctx, FZ_ERROR_GENERIC, "too many color components in bitmap");
 
+	if (end > h)
+		end = h;
+	end -= band_start;
+
 	bytestride = (w + 7) >> 3;
-	while (h--)
+	while (end--)
 	{
-		fz_write(ctx, out, p, bytestride);
+		fz_write_data(ctx, out, p, bytestride);
 		p += stride;
 	}
 }
@@ -373,21 +396,26 @@ pkm_write_band(fz_context *ctx, fz_band_writer *writer, int stride, int band_sta
 	int h = writer->h;
 	int n = writer->n;
 	int bytestride;
+	int end = band_start + band_height;
 
 	if (n != 4)
 		fz_throw(ctx, FZ_ERROR_GENERIC, "wrong number of color components in bitmap");
 
+	if (end > h)
+		end = h;
+	end -= band_start;
+
 	bytestride = stride - (w>>1);
-	while (h--)
+	while (end--)
 	{
 		int ww = w-1;
 		while (ww > 0)
 		{
-			fz_write(ctx, out, &pkm[8 * *p++], 8);
+			fz_write_data(ctx, out, &pkm[8 * *p++], 8);
 			ww -= 2;
 		}
 		if (ww == 0)
-			fz_write(ctx, out, &pkm[8 * *p], 4);
+			fz_write_data(ctx, out, &pkm[8 * *p], 4);
 		p += bytestride;
 	}
 }
@@ -402,7 +430,6 @@ fz_band_writer *fz_new_pbm_band_writer(fz_context *ctx, fz_output *out)
 	return writer;
 }
 
-
 fz_band_writer *fz_new_pkm_band_writer(fz_context *ctx, fz_output *out)
 {
 	fz_band_writer *writer = fz_new_band_writer(ctx, fz_band_writer, out);
@@ -414,7 +441,7 @@ fz_band_writer *fz_new_pkm_band_writer(fz_context *ctx, fz_output *out)
 }
 
 void
-fz_save_bitmap_as_pbm(fz_context *ctx, fz_bitmap *bitmap, char *filename)
+fz_save_bitmap_as_pbm(fz_context *ctx, fz_bitmap *bitmap, const char *filename)
 {
 	fz_output *out = fz_new_output_with_path(ctx, filename, 0);
 	fz_try(ctx)
@@ -426,7 +453,7 @@ fz_save_bitmap_as_pbm(fz_context *ctx, fz_bitmap *bitmap, char *filename)
 }
 
 void
-fz_save_bitmap_as_pkm(fz_context *ctx, fz_bitmap *bitmap, char *filename)
+fz_save_bitmap_as_pkm(fz_context *ctx, fz_bitmap *bitmap, const char *filename)
 {
 	fz_output *out = fz_new_output_with_path(ctx, filename, 0);
 	fz_try(ctx)
@@ -437,6 +464,30 @@ fz_save_bitmap_as_pkm(fz_context *ctx, fz_bitmap *bitmap, char *filename)
 		fz_rethrow(ctx);
 }
 
+void
+fz_save_pixmap_as_pbm(fz_context *ctx, fz_pixmap *pixmap, const char *filename)
+{
+	fz_bitmap *bitmap = fz_new_bitmap_from_pixmap(ctx, pixmap, NULL);
+	fz_try(ctx)
+		fz_save_bitmap_as_pbm(ctx, bitmap, filename);
+	fz_always(ctx)
+		fz_drop_bitmap(ctx, bitmap);
+	fz_catch(ctx)
+		fz_rethrow(ctx);
+}
+
+void
+fz_save_pixmap_as_pkm(fz_context *ctx, fz_pixmap *pixmap, const char *filename)
+{
+	fz_bitmap *bitmap = fz_new_bitmap_from_pixmap(ctx, pixmap, NULL);
+	fz_try(ctx)
+		fz_save_bitmap_as_pkm(ctx, bitmap, filename);
+	fz_always(ctx)
+		fz_drop_bitmap(ctx, bitmap);
+	fz_catch(ctx)
+		fz_rethrow(ctx);
+}
+
 void fz_bitmap_details(fz_bitmap *bit, int *w, int *h, int *n, int *stride)
 {
 	if (!bit)
