@@ -33,7 +33,7 @@ fz_eval_function(fz_context *ctx, fz_function *func, const float *in, int inlen,
 fz_function *
 fz_keep_function(fz_context *ctx, fz_function *func)
 {
-	return (fz_function *)fz_keep_storable(ctx, &func->storable);
+	return fz_keep_storable(ctx, &func->storable);
 }
 
 void
@@ -42,7 +42,7 @@ fz_drop_function(fz_context *ctx, fz_function *func)
 	fz_drop_storable(ctx, &func->storable);
 }
 
-unsigned int
+size_t
 fz_function_size(fz_context *ctx, fz_function *func)
 {
 	return (func ? func->size : 0);
