@@ -10,8 +10,6 @@ struct html_document_s
 	fz_document super;
 	fz_archive *zip;
 	fz_html_font_set *set;
-	float page_w, page_h, em;
-	float page_margin[4];
 	fz_html *html;
 };
 
@@ -41,8 +39,8 @@ htdoc_resolve_link(fz_context *ctx, fz_document *doc_, const char *dest, float *
 		float y = fz_find_html_target(ctx, doc->html, s+1);
 		if (y >= 0)
 		{
-			int page = y / doc->page_h;
-			if (yp) *yp = y - page * doc->page_h;
+			int page = y / doc->html->page_h;
+			if (yp) *yp = y - page * doc->html->page_h;
 			return page;
 		}
 	}
@@ -54,7 +52,7 @@ static int
 htdoc_count_pages(fz_context *ctx, fz_document *doc_)
 {
 	html_document *doc = (html_document*)doc_;
-	int count = ceilf(doc->html->root->h / doc->page_h);
+	int count = ceilf(doc->html->root->h / doc->html->page_h);
 	return count;
 }
 
@@ -63,19 +61,7 @@ htdoc_layout(fz_context *ctx, fz_document *doc_, float w, float h, float em)
 {
 	html_document *doc = (html_document*)doc_;
 
-	if (doc->html && doc->html->root)
-	{
-		doc->page_margin[T] = fz_from_css_number(doc->html->root->style.margin[T], em, em);
-		doc->page_margin[B] = fz_from_css_number(doc->html->root->style.margin[B], em, em);
-		doc->page_margin[L] = fz_from_css_number(doc->html->root->style.margin[L], em, em);
-		doc->page_margin[R] = fz_from_css_number(doc->html->root->style.margin[R], em, em);
-	}
-
-	doc->page_w = w - doc->page_margin[L] - doc->page_margin[R];
-	doc->page_h = h - doc->page_margin[T] - doc->page_margin[B];
-	doc->em = em;
-
-	fz_layout_html(ctx, doc->html, doc->page_w, doc->page_h, doc->em);
+	fz_layout_html(ctx, doc->html, w, h, em);
 }
 
 static void
@@ -90,8 +76,8 @@ htdoc_bound_page(fz_context *ctx, fz_page *page_, fz_rect *bbox)
 	html_document *doc = page->doc;
 	bbox->x0 = 0;
 	bbox->y0 = 0;
-	bbox->x1 = doc->page_w + doc->page_margin[L] + doc->page_margin[R];
-	bbox->y1 = doc->page_h + doc->page_margin[T] + doc->page_margin[B];
+	bbox->x1 = doc->html->page_w + doc->html->page_margin[L] + doc->html->page_margin[R];
+	bbox->y1 = doc->html->page_h + doc->html->page_margin[T] + doc->html->page_margin[B];
 	return bbox;
 }
 
@@ -100,12 +86,7 @@ htdoc_run_page(fz_context *ctx, fz_page *page_, fz_device *dev, const fz_matrix
 {
 	html_page *page = (html_page*)page_;
 	html_document *doc = page->doc;
-	fz_matrix local_ctm = *ctm;
-	int n = page->number;
-
-	fz_pre_translate(&local_ctm, doc->page_margin[L], doc->page_margin[T]);
-
-	fz_draw_html(ctx, dev, &local_ctm, doc->html, n * doc->page_h, (n+1) * doc->page_h);
+	fz_draw_html(ctx, dev, ctm, doc->html, page->number);
 }
 
 static fz_link *
@@ -113,27 +94,28 @@ htdoc_load_links(fz_context *ctx, fz_page *page_)
 {
 	html_page *page = (html_page*)page_;
 	html_document *doc = page->doc;
-	fz_link *head, *link;
+	return fz_load_html_links(ctx, doc->html, page->number, "", doc);
+}
 
-	head = fz_load_html_links(ctx, doc->html, page->number, doc->page_h, "");
-	for (link = head; link; link = link->next)
-	{
-		link->doc = doc;
+static fz_bookmark
+htdoc_make_bookmark(fz_context *ctx, fz_document *doc_, int page)
+{
+	html_document *doc = (html_document*)doc_;
+	return fz_make_html_bookmark(ctx, doc->html, page);
+}
 
-		/* Adjust for page margins */
-		link->rect.x0 += doc->page_margin[L];
-		link->rect.x1 += doc->page_margin[L];
-		link->rect.y0 += doc->page_margin[T];
-		link->rect.y1 += doc->page_margin[T];
-	}
-	return head;
+static int
+htdoc_lookup_bookmark(fz_context *ctx, fz_document *doc_, fz_bookmark mark)
+{
+	html_document *doc = (html_document*)doc_;
+	return fz_lookup_html_bookmark(ctx, doc->html, mark);
 }
 
 static fz_page *
 htdoc_load_page(fz_context *ctx, fz_document *doc_, int number)
 {
 	html_document *doc = (html_document*)doc_;
-	html_page *page = fz_new_page(ctx, sizeof *page);
+	html_page *page = fz_new_derived_page(ctx, html_page);
 	page->super.bound_page = htdoc_bound_page;
 	page->super.run_page_contents = htdoc_run_page;
 	page->super.load_links = htdoc_load_links;
@@ -143,7 +125,7 @@ htdoc_load_page(fz_context *ctx, fz_document *doc_, int number)
 	return (fz_page*)page;
 }
 
-int
+static int
 htdoc_lookup_metadata(fz_context *ctx, fz_document *doc_, const char *key, char *buf, int size)
 {
 	if (!strcmp(key, "format"))
@@ -157,7 +139,7 @@ htdoc_open_document_with_stream(fz_context *ctx, fz_stream *file)
 	html_document *doc;
 	fz_buffer *buf;
 
-	doc = fz_new_document(ctx, html_document);
+	doc = fz_new_derived_document(ctx, html_document);
 
 	doc->super.drop_document = htdoc_drop_document;
 	doc->super.layout = htdoc_layout;
@@ -171,18 +153,14 @@ htdoc_open_document_with_stream(fz_context *ctx, fz_stream *file)
 	doc->set = fz_new_html_font_set(ctx);
 
 	buf = fz_read_all(ctx, file, 0);
-
 	fz_try(ctx)
-	{
-		fz_write_buffer_byte(ctx, buf, 0);
 		doc->html = fz_parse_html(ctx, doc->set, doc->zip, ".", buf, fz_user_css(ctx));
-	}
 	fz_always(ctx)
 		fz_drop_buffer(ctx, buf);
 	fz_catch(ctx)
 		fz_rethrow(ctx);
 
-	return (fz_document*)doc;
+	return &doc->super;
 }
 
 static fz_document *
@@ -194,10 +172,12 @@ htdoc_open_document(fz_context *ctx, const char *filename)
 
 	fz_dirname(dirname, filename, sizeof dirname);
 
-	doc = fz_new_document(ctx, html_document);
+	doc = fz_new_derived_document(ctx, html_document);
 	doc->super.drop_document = htdoc_drop_document;
 	doc->super.layout = htdoc_layout;
 	doc->super.resolve_link = htdoc_resolve_link;
+	doc->super.make_bookmark = htdoc_make_bookmark;
+	doc->super.lookup_bookmark = htdoc_lookup_bookmark;
 	doc->super.count_pages = htdoc_count_pages;
 	doc->super.load_page = htdoc_load_page;
 	doc->super.lookup_metadata = htdoc_lookup_metadata;
@@ -207,12 +187,8 @@ htdoc_open_document(fz_context *ctx, const char *filename)
 	doc->set = fz_new_html_font_set(ctx);
 
 	buf = fz_read_file(ctx, filename);
-
 	fz_try(ctx)
-	{
-		fz_write_buffer_byte(ctx, buf, 0);
 		doc->html = fz_parse_html(ctx, doc->set, doc->zip, ".", buf, fz_user_css(ctx));
-	}
 	fz_always(ctx)
 		fz_drop_buffer(ctx, buf);
 	fz_catch(ctx)
@@ -240,7 +216,7 @@ htdoc_recognize(fz_context *doc, const char *magic)
 
 fz_document_handler html_document_handler =
 {
-	&htdoc_recognize,
-	&htdoc_open_document,
-	&htdoc_open_document_with_stream
+	htdoc_recognize,
+	htdoc_open_document,
+	htdoc_open_document_with_stream
 };
